@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapcrypto.c 7544 2008-04-29 18:16:05Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  Encryption functions (see MS-RFC-18)
@@ -33,7 +34,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapcrypto.c 6373 2007-07-24 02:37:18Z dmorissette $")
+MS_CVSID("$Id: mapcrypto.c 7544 2008-04-29 18:16:05Z pramsey $")
 
 
 /**********************************************************************
@@ -602,7 +603,7 @@ int main(int argc, char *argv[])
     {
         char *pszBuf;
         mapObj *map;
-        //map = msNewMapObj();
+        /* map = msNewMapObj(); */
         map = msLoadMap("/tmp/test.map", NULL);
 
         sprintf(string_buf2, "string with a {%s} encrypted token", string_buf);
