@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapdrawgdal.c 9496 2009-10-20 03:26:07Z warmerdam $
+ * $Id: mapdrawgdal.c 10012 2010-03-25 14:42:48Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Code for drawing GDAL raster layers.  Called from 
@@ -33,7 +33,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c 9496 2009-10-20 03:26:07Z warmerdam $")
+MS_CVSID("$Id: mapdrawgdal.c 10012 2010-03-25 14:42:48Z warmerdam $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -1613,7 +1613,8 @@ static void Dither24to8( GByte *pabyRed, GByte *pabyGreen, GByte *pabyBlue,
 /*                       msGetGDALGeoTransform()                        */
 /*                                                                      */
 /*      Cover function that tries GDALGetGeoTransform(), a world        */
-/*      file or OWS extents.                                            */
+/*      file or OWS extents.  It is assumed that TLOCK_GDAL is held     */
+/*      before this function is called.                                 */
 /************************************************************************/
 
 int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer, 
@@ -1621,9 +1622,8 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
 
 {
     const char *extent_priority = NULL;
-#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
+    const char *value;
     rectObj  rect;
-#endif
 
 /* -------------------------------------------------------------------- */
 /*      some GDAL drivers (ie. GIF) don't set geotransform on failure.  */
@@ -1644,7 +1644,7 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
     if( extent_priority != NULL
         && EQUALN(extent_priority,"WORLD",5) )
     {
-	        if( GDALGetDescription(hDS) != NULL 
+        if( GDALGetDescription(hDS) != NULL 
             && GDALReadWorldFile(GDALGetDescription(hDS), "wld", 
                                  padfGeoTransform) )
         {
@@ -1675,19 +1675,25 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
 /* -------------------------------------------------------------------- */
 /*      Try worldfile.                                                  */
 /* -------------------------------------------------------------------- */
-    else if( GDALGetDescription(hDS) != NULL 
-             && GDALReadWorldFile(GDALGetDescription(hDS), "wld", 
-                                  padfGeoTransform) )
+    if( GDALGetDescription(hDS) != NULL 
+        && GDALReadWorldFile(GDALGetDescription(hDS), "wld", 
+                             padfGeoTransform) )
     {
         return MS_SUCCESS;
     }
 
 /* -------------------------------------------------------------------- */
-/*      Try OWS extent metadata.                                        */
+/*      Do we have the extent keyword on the layer?  We only use        */
+/*      this if we have a single raster associated with the layer as    */
+/*      opposed to a tile index.                                        */
+/*                                                                      */
+/*      Arguably this ought to take priority over all the other         */
+/*      stuff.                                                          */
 /* -------------------------------------------------------------------- */
-#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
-    else if( msOWSGetLayerExtent( map, layer, "MFCO", &rect ) == MS_SUCCESS )
+    if (MS_VALID_EXTENT(layer->extent) && layer->data != NULL)
     {
+        rect = layer->extent;
+
         padfGeoTransform[0] = rect.minx;
         padfGeoTransform[1] = (rect.maxx - rect.minx) /
             (double) GDALGetRasterXSize( hDS );
@@ -1696,9 +1702,40 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
         padfGeoTransform[4] = 0;
         padfGeoTransform[5] = (rect.miny - rect.maxy) /
             (double) GDALGetRasterYSize( hDS );
-
+        
         return MS_SUCCESS;
     }
+
+/* -------------------------------------------------------------------- */
+/*      Try OWS extent metadata.  We only try this if we know there     */
+/*      is metadata so that we don't end up going into the layer        */
+/*      getextent function which will in turn reopen the file with      */
+/*      potential performance and locking problems.                     */
+/* -------------------------------------------------------------------- */
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
+    if ((value = msOWSLookupMetadata(&(layer->metadata), "MFCO", "extent"))
+        != NULL)
+    {
+        int success;
+
+        msReleaseLock( TLOCK_GDAL );
+        success = msOWSGetLayerExtent( map, layer, "MFCO", &rect );
+        msAcquireLock( TLOCK_GDAL );
+
+        if( success == MS_SUCCESS )
+        {
+            padfGeoTransform[0] = rect.minx;
+            padfGeoTransform[1] = (rect.maxx - rect.minx) /
+                (double) GDALGetRasterXSize( hDS );
+            padfGeoTransform[2] = 0;
+            padfGeoTransform[3] = rect.maxy;
+            padfGeoTransform[4] = 0;
+            padfGeoTransform[5] = (rect.miny - rect.maxy) /
+                (double) GDALGetRasterYSize( hDS );
+            
+            return MS_SUCCESS;
+        }
+    }
 #endif
 
 /* -------------------------------------------------------------------- */
@@ -1706,17 +1743,14 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
 /*      Reset our default geotransform.  GDALGetGeoTransform() may      */
 /*      have altered it even if GDALGetGeoTransform() failed.           */
 /* -------------------------------------------------------------------- */
-    else
-    {
-        padfGeoTransform[0] = 0.0;
-        padfGeoTransform[1] = 1.0;
-        padfGeoTransform[2] = 0.0;
-        padfGeoTransform[3] = GDALGetRasterYSize(hDS);
-        padfGeoTransform[4] = 0.0;
-        padfGeoTransform[5] = -1.0;
-
-        return MS_FAILURE;
-    }
+    padfGeoTransform[0] = 0.0;
+    padfGeoTransform[1] = 1.0;
+    padfGeoTransform[2] = 0.0;
+    padfGeoTransform[3] = GDALGetRasterYSize(hDS);
+    padfGeoTransform[4] = 0.0;
+    padfGeoTransform[5] = -1.0;
+    
+    return MS_FAILURE;
 }
 
 /************************************************************************/
