@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapgml.c 9455 2009-10-14 04:42:49Z dmorissette $
+ * $Id: mapgml.c 9887 2010-02-23 19:30:57Z assefa $
  *
  * Project:  MapServer
  * Purpose:  shapeObj to GML output via MapServer queries.
@@ -31,7 +31,7 @@
 #include "maperror.h"
 #include "mapgml.h"
 
-MS_CVSID("$Id: mapgml.c 9455 2009-10-14 04:42:49Z dmorissette $")
+MS_CVSID("$Id: mapgml.c 9887 2010-02-23 19:30:57Z assefa $")
 
 /* Use only mapgml.c if WMS or WFS is available */
 
@@ -1413,8 +1413,11 @@ void msAxisSwapShape(shapeObj *shape)
 ** msGMLWriteWFSQuery()
 **
 ** Similar to msGMLWriteQuery() but tuned for use with WFS 1.0.0
+** bUseGetShape is used when the query passed through a commplex filter
+** encoding #3305
 */
-int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default_namespace_prefix, int outputformat)
+int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default_namespace_prefix, 
+                       int outputformat, int bUseGetShape)
 {
 #ifdef USE_WFS_SVR
   int status;
@@ -1510,7 +1513,11 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
       }
 
       for(j=0; j<lp->resultcache->numresults; j++) {
-        status = msLayerResultsGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
+        if (bUseGetShape)
+          status = msLayerGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);  
+        else
+          status =  msLayerResultsGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);  
+
         if(status != MS_SUCCESS) 
             return(status);
 
