@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maphash.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: maphash.c 9376 2009-10-05 14:09:14Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Implement hashTableObj class.
@@ -32,7 +32,7 @@
 #include "mapserver.h"
 #include "maphash.h"
 
-MS_CVSID("$Id: maphash.c 7418 2008-02-29 00:02:49Z nsavard $")
+MS_CVSID("$Id: maphash.c 9376 2009-10-05 14:09:14Z aboudreault $")
 
 static unsigned hash(const char *key)
 {
@@ -197,7 +197,7 @@ int msRemoveHashTable(hashTableObj *table, const char *key)
                 break;
             }
             else {
-                table->items[hash(key)] = NULL;
+                table->items[hash(key)] = tp->next;
                 free(tp);
                 break;
             }
