@@ -1,8 +1,11 @@
 /******************************************************************************
+ * $Id: mapowscommon.c 8170 2008-12-02 17:28:34Z tomkralidis $
  *
- * Name:     mapowscommon.c
  * Project:  MapServer
  * Purpose:  OGC OWS Common Implementation for use by MapServer OGC code
+ *           versions:
+ *           1.0.0 (OGC Document 05-008c1)
+ *           1.1.0 (OGC document 06-121r3)
  *           
  * Author:   Tom Kralidis (tomkralidis@hotmail.com)
  *
@@ -29,20 +32,56 @@
  ****************************************************************************/
 
 #include "mapserver.h"
+#include "mapows.h"
 
-#ifdef USE_SOS_SVR
+#ifdef USE_LIBXML2
 
 #include<libxml/parser.h>
 #include<libxml/tree.h>
 
 #include "mapowscommon.h"
+#include "maplibxml2.h"
+
+MS_CVSID("$Id: mapowscommon.c 8170 2008-12-02 17:28:34Z tomkralidis $")
+
+/**
+ * msOWSCommonNegotiateVersion()
+ *
+ * returns a supported version as per subclause 7.3.2
+ *
+ * @param requested_version the version passed by the client
+ * @param supported_versions an array of supported versions
+ * @param num_supported_versions size of supported_versions
+ *
+ * @return supported version integer, or -1 on error
+ *
+ */
+
+int msOWSCommonNegotiateVersion(int requested_version, int supported_versions[], int num_supported_versions) {
+  int i;
+
+  /* if version is not set return error */
+  if (! requested_version)
+    return -1;
+
+  /* return the first entry that's equal to the requested version */
+  for (i = 0; i < num_supported_versions; i++) {
+    if (supported_versions[i] == requested_version)
+      return supported_versions[i];
+  }
+
+  /* no match; calling code should throw an exception */
+  return -1;
+}
 
-MS_CVSID("$Id: mapowscommon.c 6985 2007-10-31 17:49:25Z tomkralidis $")
 
 /**
  * msOWSCommonServiceIdentification()
  *
- * returns an object of ServiceIdentification as per subclause 7.4.3
+ * returns an object of ServiceIdentification as per:
+ *
+ * 1.0.0 subclause 7.4.3
+ * 1.1.1 subclause 7.4.4
  *
  * @param map mapObj used to fetch WEB/METADATA
  * @param servicetype the OWS type
@@ -52,77 +91,64 @@ MS_CVSID("$Id: mapowscommon.c 6985 2007-10-31 17:49:25Z tomkralidis $")
  *
  */
 
-xmlNodePtr msOWSCommonServiceIdentification(mapObj *map, const char *servicetype, const char *version) {
+xmlNodePtr msOWSCommonServiceIdentification(xmlNsPtr psNsOws, mapObj *map, const char *servicetype, const char *version, const char *namespaces) {
   const char *value    = NULL;
 
-  xmlNsPtr     psNs       = NULL;
   xmlNodePtr   psRootNode = NULL;
   xmlNodePtr   psNode     = NULL;
-  xmlNodePtr   psSubNode  = NULL;
 
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
   /* create element name */
-  psRootNode = xmlNewNode(xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX), BAD_CAST "ServiceIdentification");
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST "ServiceIdentification");
 
   /* add child elements */
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "title");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "title");
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "Title", BAD_CAST value);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "Title", BAD_CAST value);
 
   if (!value) {
     xmlAddSibling(psNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_title\" missing for ows:Title"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "abstract");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "abstract");
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "Abstract", BAD_CAST value);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "Abstract", BAD_CAST value);
 
   if (!value) {
     xmlAddSibling(psNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_abstract\" was missing for ows:Abstract"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "keywordlist");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "keywordlist");
 
   if (value) {
-    char **tokens = NULL;
-    int n = 0;
-    int i = 0;
-
-    psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "Keywords", NULL);
-
-    tokens = msStringSplit(value, ',', &n);
-    if (tokens && n > 0) {
-      for (i=0; i<n; i++) {
-        psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "Keyword", BAD_CAST tokens[i]);
-        xmlSetNs(psSubNode, psNs);
-      }
-      msFreeCharArray(tokens, n);
-    }
+    psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "Keywords", NULL);
+    msLibXml2GenerateList(psNode, psNsOws, "Keyword", value, ',');
   }
 
   else {
     xmlAddSibling(psNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_keywordlist\" was missing for ows:KeywordList"));
   }
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "ServiceType", BAD_CAST servicetype);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "ServiceType", BAD_CAST servicetype);
   
   xmlNewProp(psNode, BAD_CAST "codeSpace", BAD_CAST MS_OWSCOMMON_OGC_CODESPACE);
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "ServiceTypeVersion", BAD_CAST version);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "ServiceTypeVersion", BAD_CAST version);
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "fees");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "fees");
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "Fees", BAD_CAST value);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "Fees", BAD_CAST value);
 
   if (!value) {
     xmlAddSibling(psNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_fees\" was missing for ows:Fees"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "accessconstraints");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "accessconstraints");
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "AccessConstraints", BAD_CAST value);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "AccessConstraints", BAD_CAST value);
 
   if (!value) {
     xmlAddSibling(psNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_accessconstraints\" was missing for ows:AccessConstraints"));
@@ -134,7 +160,11 @@ xmlNodePtr msOWSCommonServiceIdentification(mapObj *map, const char *servicetype
 /**
  * msOWSCommonServiceProvider()
  *
- * returns an object of ServiceProvider as per subclause 7.4.4
+ * returns an object of ServiceProvider as per:
+ * 
+ *
+ * 1.0.0 subclause 7.4.4
+ * 1.1.0 subclause 7.4.5
  *
  * @param map mapObj to fetch MAP/WEB/METADATA
  *
@@ -142,26 +172,24 @@ xmlNodePtr msOWSCommonServiceIdentification(mapObj *map, const char *servicetype
  *
  */
 
-xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
+xmlNodePtr msOWSCommonServiceProvider(xmlNsPtr psNsOws, xmlNsPtr psNsXLink,
+                                      mapObj *map, const char *namespaces) {
   const char *value = NULL;
 
-  xmlNsPtr     psNsXLink       = NULL;
-  xmlNsPtr     psNsOws         = NULL;
   xmlNodePtr   psNode          = NULL;
   xmlNodePtr   psRootNode      = NULL;
   xmlNodePtr   psSubNode       = NULL;
   xmlNodePtr   psSubSubNode    = NULL;
   xmlNodePtr   psSubSubSubNode = NULL;
 
-  psNsXLink = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_PREFIX);
-
-  psNsOws = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
   psRootNode = xmlNewNode(psNsOws, BAD_CAST "ServiceProvider");
 
   /* add child elements */
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactorganization");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactorganization");
 
   psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "ProviderName", BAD_CAST value);
 
@@ -173,7 +201,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 
   xmlNewNsProp(psNode, psNsXLink, BAD_CAST "type", BAD_CAST "simple");
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "service_onlineresource");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "service_onlineresource");
 
   xmlNewNsProp(psNode, psNsXLink, BAD_CAST "href", BAD_CAST value);
 
@@ -183,7 +211,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 
   psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "ServiceContact", NULL);
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactperson");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactperson");
 
   psSubNode = xmlNewChild(psNode, psNsOws, BAD_CAST "IndividualName", BAD_CAST  value);
 
@@ -191,7 +219,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_contactperson\" was missing for ows:IndividualName"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactposition");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactposition");
 
   psSubNode = xmlNewChild(psNode, psNsOws, BAD_CAST "PositionName", BAD_CAST value); 
 
@@ -203,7 +231,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 
   psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "Phone", NULL);
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactvoicetelephone");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactvoicetelephone");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "Voice", BAD_CAST value);
 
@@ -211,7 +239,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_contactvoicetelephone\" was missing for ows:Voice"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactfacsimiletelephone");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactfacsimiletelephone");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "Facsimile", BAD_CAST value);
 
@@ -221,7 +249,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 
   psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "Address", NULL);
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "address");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "address");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "DeliveryPoint", BAD_CAST value);
 
@@ -229,7 +257,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_address\" was missing for ows:DeliveryPoint"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "city");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "city");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "City", BAD_CAST value);
 
@@ -237,7 +265,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_city\" was missing for ows:City"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "stateorprovince");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "stateorprovince");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "AdministrativeArea", BAD_CAST value);
 
@@ -245,7 +273,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_stateorprovince\" was missing for ows:AdministrativeArea"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "postcode");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "postcode");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "PostalCode", BAD_CAST value);
 
@@ -253,7 +281,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_postcode\" was missing for ows:PostalCode"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "country");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "country");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "Country", BAD_CAST value);
 
@@ -261,7 +289,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_country\" was missing for ows:Country"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactelectronicmailaddress");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactelectronicmailaddress");
 
   psSubSubSubNode = xmlNewChild(psSubSubNode, psNsOws, BAD_CAST "ElectronicMailAddress", BAD_CAST value);
 
@@ -273,7 +301,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 
   xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "type", BAD_CAST "simple");
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "service_onlineresource");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "service_onlineresource");
 
   xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "href", BAD_CAST value);
 
@@ -281,7 +309,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_service_onlineresource\" was missing for ows:OnlineResource/@xlink:href"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "hoursofservice");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "hoursofservice");
 
   psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "HoursOfService", BAD_CAST value);
 
@@ -289,7 +317,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_hoursofservice\" was missing for ows:HoursOfService"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "contactinstructions");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "contactinstructions");
 
   psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "ContactInstructions", BAD_CAST value);
 
@@ -297,7 +325,7 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
     xmlAddSibling(psSubSubNode, xmlNewComment(BAD_CAST "WARNING: Optional metadata \"ows_contactinstructions\" was missing for ows:ContactInstructions"));
   }
 
-  value = msOWSLookupMetadata(&(map->web.metadata), "O", "role");
+  value = msOWSLookupMetadata(&(map->web.metadata), namespaces, "role");
 
   psSubNode = xmlNewChild(psNode, psNsOws, BAD_CAST "Role", BAD_CAST value);
 
@@ -312,112 +340,110 @@ xmlNodePtr msOWSCommonServiceProvider(mapObj *map) {
 /**
  * msOWSCommonOperationsMetadata()
  *
- * returns the root element of OperationsMetadata as per subclause 7.4.5
+ * returns the root element of OperationsMetadata as per:
+ *
+ * 1.0.0 subclause 7.4.5
+ * 1.1.0 subclause 7.4.6
  *
  * @return psRootNode xmlNodePtr pointer of XML construct
  *
  */
 
-xmlNodePtr msOWSCommonOperationsMetadata() {
-  xmlNsPtr psNs         = NULL;
+xmlNodePtr msOWSCommonOperationsMetadata(xmlNsPtr psNsOws) {
   xmlNodePtr psRootNode = NULL;
 
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
-  psRootNode = xmlNewNode(psNs, BAD_CAST "OperationsMetadata");
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST "OperationsMetadata");
   return psRootNode;
 }
 
 /**
  * msOWSCommonOperationsMetadataOperation()
  *
- * returns an Operation element of OperationsMetadata as per subclause 7.4.5
+ * returns an Operation element of OperationsMetadata as per:
+ * 
+ * 1.0.0 subclause 7.4.5
+ * 1.1.0 subclause 7.4.6
  *
  * @param name name of the Operation
- * @param method HTTP method (1 for GET only, 2 for POST only, 3 for GET and POST)
+ * @param method HTTP method: OWS_METHOD_GET, OWS_METHOD_POST or OWS_METHOD_GETPOST)
  * @param url online resource URL
  *
  * @return psRootNode xmlNodePtr pointer of XML construct
- *
  */
 
-xmlNodePtr msOWSCommonOperationsMetadataOperation(char *name, int method, char *url) {
-  xmlNsPtr   psNs            = NULL;
-  xmlNsPtr   psNsXLink       = NULL;
+xmlNodePtr msOWSCommonOperationsMetadataOperation(xmlNsPtr psNsOws, xmlNsPtr psXLinkNs, char *name, int method, char *url) {
   xmlNodePtr psRootNode      = NULL;
   xmlNodePtr psNode          = NULL;
   xmlNodePtr psSubNode       = NULL;
   xmlNodePtr psSubSubNode    = NULL;
 
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
-  psNsXLink = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_PREFIX);
 
-  psRootNode = xmlNewNode(psNs, BAD_CAST "Operation");
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST "Operation");
 
   xmlNewProp(psRootNode, BAD_CAST "name", BAD_CAST name);
 
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "DCP", NULL);
+  psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "DCP", NULL);
 
-  psSubNode = xmlNewChild(psNode, psNs, BAD_CAST "HTTP", NULL);
+  psSubNode = xmlNewChild(psNode, psNsOws, BAD_CAST "HTTP", NULL);
 
-  if (method  == 1 || method == 3) {
-    psSubSubNode = xmlNewChild(psSubNode, psNs, BAD_CAST "Get", NULL);
-    xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "type", BAD_CAST "simple");
-    xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "href", BAD_CAST url);
+  if (method  == OWS_METHOD_GET || method == OWS_METHOD_GETPOST ) {
+    psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "Get", NULL);
+    xmlNewNsProp(psSubSubNode, psXLinkNs, BAD_CAST "type", BAD_CAST "simple");
+    xmlNewNsProp(psSubSubNode, psXLinkNs, BAD_CAST "href", BAD_CAST url);
   }
 
-  if (method == 2 || method == 3) {
-    psSubSubNode = xmlNewChild(psSubNode, psNs, BAD_CAST "Post", NULL);
-    xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "type", BAD_CAST "simple");
-    xmlNewNsProp(psSubSubNode, psNsXLink, BAD_CAST "href", BAD_CAST url);
+  if (method == OWS_METHOD_POST || method == OWS_METHOD_GETPOST ) {
+    psSubSubNode = xmlNewChild(psSubNode, psNsOws, BAD_CAST "Post", NULL);
+    xmlNewNsProp(psSubSubNode, psXLinkNs, BAD_CAST "type", BAD_CAST "simple");
+    xmlNewNsProp(psSubSubNode, psXLinkNs, BAD_CAST "href", BAD_CAST url);
   }
 
   return psRootNode;
 }
 
 /**
- * msOWSCommonOperationsMetadataParameter()
+ * msOWSCommonOperationsMetadataDomainType()
  *
- * returns an Parameter element of OperationsMetadata as per subclause 7.4.5
+ * returns a Parameter or Constraint element (which are of type ows:DomainType)
+ * of OperationsMetadata as per:
  *
+ * 1.0.0 subclause 7.4.5
+ * 1.1.0 subclause 7.4.6
+ *
+ * @param version the integerized x.y.z version of OWS Common to use
+ * @param elname name of the element (Parameter | Constraint)
  * @param name name of the Parameter
- * @param use use policy for Parameter (0 for optional, 1 for required)
  * @param values list of values (comma seperated list) or NULL if none
  *
  * @return psRootNode xmlNodePtr pointer of XML construct
  *
  */
 
-xmlNodePtr msOWSCommonOperationsMetadataParameter(char *name, int use, char *values) {
-  xmlNsPtr   psNs       = NULL;
+xmlNodePtr msOWSCommonOperationsMetadataDomainType(int version, xmlNsPtr psNsOws, char *elname, char *name, char *values) {
   xmlNodePtr psRootNode = NULL;
-  xmlNodePtr psNode     = NULL;
+  xmlNodePtr psNode = NULL;
 
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
-  psRootNode = xmlNewNode(psNs, BAD_CAST "Parameter");
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST elname);
 
   xmlNewProp(psRootNode, BAD_CAST "name", BAD_CAST name);
 
-  if (use == 1) {
-    xmlNewProp(psRootNode, BAD_CAST "use", BAD_CAST "required");
-  } 
-  else {
-    xmlNewProp(psRootNode, BAD_CAST "use", BAD_CAST "optional");
-  } 
-
-  if (values != NULL) {
-    char **tokens = NULL;
-    int n = 0;
-    int i = 0;
-    tokens = msStringSplit(values, ',', &n);
-    if (tokens && n > 0) {
-      for (i=0; i<n; i++) {
-        psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "Value", BAD_CAST tokens[i]);
-      }
-      msFreeCharArray(tokens, n);
-    }
+  if (version == OWS_1_0_0) {
+    msLibXml2GenerateList(psRootNode, psNsOws, "Value", values, ',');
   }
+  if (version == OWS_1_1_0) {
+    psNode = xmlNewChild(psRootNode, psNsOws, BAD_CAST "AllowedValues", NULL);
+    msLibXml2GenerateList(psNode, psNsOws, "Value", values, ',');
+  }
+
   return psRootNode;
 }
 
@@ -426,6 +452,7 @@ xmlNodePtr msOWSCommonOperationsMetadataParameter(char *name, int use, char *val
  *
  * returns an object of ExceptionReport as per clause 8
  *
+ * @param ows_version the version of OWS Common to use
  * @param schemas_location URL to OGC Schemas Location base
  * @param version the version of the calling specification
  * @param language ISO3166 code of language
@@ -437,34 +464,35 @@ xmlNodePtr msOWSCommonOperationsMetadataParameter(char *name, int use, char *val
  *
  */
 
-xmlNodePtr msOWSCommonExceptionReport(const char *schemas_location, const char *version, const char *language, const char *exceptionCode, const char *locator, const char *ExceptionText) {
+xmlNodePtr msOWSCommonExceptionReport(xmlNsPtr psNsOws, int ows_version, const char *schemas_location, const char *version, const char *language, const char *exceptionCode, const char *locator, const char *ExceptionText) {
   char *xsi_schemaLocation = NULL;
+  char szVersionBuf[OWS_VERSION_MAXLEN];
 
   xmlNsPtr     psNsXsi     = NULL;
-  xmlNsPtr     psNsOws     = NULL;
   xmlNodePtr   psRootNode  = NULL;
   xmlNodePtr   psMainNode  = NULL;
   xmlNodePtr   psNode      = NULL;
 
-  psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
-
   psRootNode = xmlNewNode(psNsOws, BAD_CAST "ExceptionReport");
 
   psNsXsi = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_PREFIX);
 
-  xmlSetNs(psRootNode,  psNsOws);
-
   /* add attributes to root element */
   xmlNewProp(psRootNode, BAD_CAST "version", BAD_CAST version);
 
-  if (language !=  "undefined") {
+  if (ows_version == OWS_1_0_0) {
     xmlNewProp(psRootNode, BAD_CAST "language", BAD_CAST language);
   }
+  if (ows_version == OWS_1_1_0) {
+    xmlNewProp(psRootNode, BAD_CAST "xml:lang", BAD_CAST language);
+  }
 
-  xsi_schemaLocation = strdup(MS_OWSCOMMON_OWS_NAMESPACE_URI);
+  xsi_schemaLocation = strdup((char *)psNsOws->href);
   xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, " ");
   xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, (char *)schemas_location);
-  xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/ows/1.0.0/owsExceptionReport.xsd");
+  xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/ows/");
+  xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, (char *)msOWSGetVersionString(ows_version, szVersionBuf));
+  xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/owsExceptionReport.xsd");
 
   /* add namespace'd attributes to root element */
   xmlNewNsProp(psRootNode, psNsXsi, BAD_CAST "schemaLocation", BAD_CAST xsi_schemaLocation);
@@ -483,7 +511,7 @@ xmlNodePtr msOWSCommonExceptionReport(const char *schemas_location, const char *
     psNode = xmlNewChild(psMainNode, NULL, BAD_CAST "ExceptionText", BAD_CAST ExceptionText);
   }
 
-  xmlFreeNs(psNsOws);
+  free(xsi_schemaLocation);
   return psRootNode;
 }
 
@@ -492,6 +520,10 @@ xmlNodePtr msOWSCommonExceptionReport(const char *schemas_location, const char *
  *
  * returns an object of BoundingBox as per subclause 10.2.1
  *
+ * If necessary (ie. an EPSG URN GCS such as 4326) the tuple axes will be
+ * reoriented to match the EPSG coordinate system expectations.
+ *
+ * @param psNsOws OWS namespace object
  * @param crs the CRS / EPSG code
  * @param dimensions number of dimensions of the coordinates
  * @param minx minx
@@ -500,22 +532,33 @@ xmlNodePtr msOWSCommonExceptionReport(const char *schemas_location, const char *
  * @param maxy maxy
  *
  * @return psRootNode xmlNodePtr pointer of XML construct
- *
  */
 
-xmlNodePtr msOWSCommonBoundingBox(const char *crs, int dimensions, double minx, double miny, double maxx, double maxy) {
+xmlNodePtr msOWSCommonBoundingBox(xmlNsPtr psNsOws, const char *crs, int dimensions, double minx, double miny, double maxx, double maxy) {
   char LowerCorner[100];
   char UpperCorner[100];
   char dim_string[100];
-
-  xmlNsPtr   psNs       = NULL;
   xmlNodePtr psRootNode = NULL;
-  xmlNodePtr psNode     = NULL;
 
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  /* Do we need to reorient tuple axes? */
+  {
+      projectionObj proj;
+
+      msInitProjection( &proj );
+      if( msLoadProjectionString( &proj, (char *) crs ) == 0 )
+      {
+          msAxisNormalizePoints( &proj, 1, &minx, &miny );
+          msAxisNormalizePoints( &proj, 1, &maxx, &maxy );
+      }
+      msFreeProjection( &proj );
+  }
+
+
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
   /* create element name */
-  psRootNode = xmlNewNode(psNs, BAD_CAST "BoundingBox");
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST "BoundingBox");
 
   /* add attributes to the root element */
   xmlNewProp(psRootNode, BAD_CAST "crs", BAD_CAST crs);
@@ -523,12 +566,12 @@ xmlNodePtr msOWSCommonBoundingBox(const char *crs, int dimensions, double minx,
   sprintf( dim_string, "%d", dimensions );
   xmlNewProp(psRootNode, BAD_CAST "dimensions", BAD_CAST dim_string);
 
-  sprintf(LowerCorner, "%g %g", minx, miny);
-  sprintf(UpperCorner, "%g %g", maxx, maxy);
+  sprintf(LowerCorner, "%.15g %.15g", minx, miny);
+  sprintf(UpperCorner, "%.15g %.15g", maxx, maxy);
 
   /* add child elements */
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "LowerCorner", BAD_CAST LowerCorner);
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "UpperCorner", BAD_CAST UpperCorner);
+  xmlNewChild(psRootNode, psNsOws,BAD_CAST "LowerCorner",BAD_CAST LowerCorner);
+  xmlNewChild(psRootNode, psNsOws,BAD_CAST "UpperCorner",BAD_CAST UpperCorner);
 
   return psRootNode;
 }
@@ -536,8 +579,9 @@ xmlNodePtr msOWSCommonBoundingBox(const char *crs, int dimensions, double minx,
 /**
  * msOWSCommonWGS84BoundingBox()
  *
- * returns an object of BoundingBox as per subclause 10.2.2
+ * returns an object of WGS84BoundingBox as per subclause 10.2.2
  *
+ * @param psNsOws OWS namespace object
  * @param dimensions number of dimensions of the coordinates
  * @param minx minx
  * @param miny miny
@@ -545,35 +589,52 @@ xmlNodePtr msOWSCommonBoundingBox(const char *crs, int dimensions, double minx,
  * @param maxy maxy
  *
  * @return psRootNode xmlNodePtr pointer of XML construct
- *
  */
 
-xmlNodePtr msOWSCommonWGS84BoundingBox(int dimensions, double minx, double miny, double maxx, double maxy) {
+xmlNodePtr msOWSCommonWGS84BoundingBox(xmlNsPtr psNsOws, int dimensions, double minx, double miny, double maxx, double maxy) {
   char LowerCorner[100];
   char UpperCorner[100];
   char dim_string[100];
 
-  xmlNsPtr   psNs       = NULL;
   xmlNodePtr psRootNode = NULL;
-  xmlNodePtr psNode     = NULL;
-
-  psNs = xmlNewNs(NULL, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+  
+  if (_validateNamespace(psNsOws) == MS_FAILURE)
+    psNsOws = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
 
   /* create element name */
-  psRootNode = xmlNewNode(psNs, BAD_CAST "WGS84BoundingBox");
+  psRootNode = xmlNewNode(psNsOws, BAD_CAST "WGS84BoundingBox");
 
   sprintf( dim_string, "%d", dimensions );
   xmlNewProp(psRootNode, BAD_CAST "dimensions", BAD_CAST dim_string);
 
-  sprintf(LowerCorner, "%g %g", minx, miny);
-  sprintf(UpperCorner, "%g %g", maxx, maxy);
+  sprintf(LowerCorner, "%.15g %.15g", minx, miny);
+  sprintf(UpperCorner, "%.15g %.15g", maxx, maxy);
 
   /* add child elements */
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "LowerCorner", BAD_CAST LowerCorner);
-  psNode = xmlNewChild(psRootNode, psNs, BAD_CAST "UpperCorner", BAD_CAST UpperCorner);
+  xmlNewChild(psRootNode, psNsOws,BAD_CAST "LowerCorner",BAD_CAST LowerCorner);
+  xmlNewChild(psRootNode, psNsOws,BAD_CAST "UpperCorner",BAD_CAST UpperCorner);
 
   return psRootNode;
 }
 
-#endif /* defined(USE_SOS_SVR) */
+/**
+ * _validateNamespace()
+ *
+ * validates the namespace passed to this module's functions
+ *
+ * @param psNsOws namespace object
+ *
+ * @return MS_SUCCESS or MS_FAILURE
+ *
+ */
 
+int _validateNamespace(xmlNsPtr psNsOws) {
+  char namespace_prefix[10];
+  sprintf(namespace_prefix, "%s", psNsOws->prefix);
+  if (strcmp(namespace_prefix, MS_OWSCOMMON_OWS_NAMESPACE_PREFIX) == 0)
+    return MS_SUCCESS;
+  else 
+    return MS_FAILURE;
+}
+
+#endif /* defined(USE_LIBXML2) */
