@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapserv.h 6428 2007-07-31 19:58:45Z dmorissette $
+ * $Id: mapserv.h 7549 2008-04-30 20:50:31Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Declarations supporting mapserv.c.
@@ -38,6 +38,7 @@
 #include <time.h>
 
 #include "maptemplate.h"
+#include "maptile.h"
 
 #include "cgiutil.h"
 /*
@@ -56,13 +57,13 @@ extern int enter_string;
 /*
 ** Enumerated types, keep the query modes in sequence and at the end of the enumeration (mode enumeration is in maptemplate.h).
 */
-int numModes = 26;
-static char *modeStrings[26] = {"BROWSE","ZOOMIN","ZOOMOUT","MAP","LEGEND","REFERENCE","SCALEBAR","COORDINATE",
+int numModes = 29;
+static char *modeStrings[29] = {"BROWSE","ZOOMIN","ZOOMOUT","MAP","LEGEND","LEGENDICON","REFERENCE","SCALEBAR","COORDINATE",
                                 "QUERY","QUERYMAP","NQUERY","NQUERYMAP",
 			        "ITEMQUERY","ITEMQUERYMAP","ITEMNQUERY","ITEMNQUERYMAP",
 				"FEATUREQUERY","FEATUREQUERYMAP","FEATURENQUERY","FEATURENQUERYMAP",
 				"ITEMFEATUREQUERY","ITEMFEATUREQUERYMAP","ITEMFEATURENQUERY","ITEMFEATURENQUERYMAP",
-				"INDEXQUERY","INDEXQUERYMAP"};
+				"INDEXQUERY","INDEXQUERYMAP","TILE","OWS"};
 
 /*
 ** Global variables
