@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapstring.c 8268 2008-12-22 14:48:10Z aboudreault $
+ * $Id: mapstring.c 9999 2010-03-24 15:26:31Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Various string handling functions.
@@ -33,10 +33,11 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapstring.c 8268 2008-12-22 14:48:10Z aboudreault $")
+MS_CVSID("$Id: mapstring.c 9999 2010-03-24 15:26:31Z warmerdam $")
 
 #include <ctype.h>
 #include <string.h>
+#include <errno.h>
 
 /*
  * Find the first occurrence of find in s, ignore case.
@@ -1446,7 +1447,7 @@ char *msGetEncodedString(const char *string, const char *encoding)
   iconv_t cd = NULL;
   const char *inp;
   char *outp, *out = NULL;
-  size_t len, bufsize, bufleft, status;
+  size_t len, bufsize, bufleft, iconv_status;
 
 #ifdef USE_FRIBIDI
   if(fribidi_parse_charset ((char*)encoding))
@@ -1476,11 +1477,11 @@ char *msGetEncodedString(const char *string, const char *encoding)
   outp = out;
 
   bufleft = bufsize;
-  status = -1;
+  iconv_status = -1;
 
   while (len > 0){
-    status = iconv(cd, (char**)&inp, &len, &outp, &bufleft);
-    if(status == -1){
+    iconv_status = iconv(cd, (char**)&inp, &len, &outp, &bufleft);
+    if(iconv_status == -1){
       msFree(out);
       iconv_close(cd);
       return strdup(string);
@@ -1504,13 +1505,13 @@ char *msGetEncodedString(const char *string, const char *encoding)
 char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
 #ifdef USE_ICONV
 
-    int bconvFailed = MS_TRUE;
     char* output = NULL;
+    char* errormessage = NULL;
     iconv_t cd = NULL;
     size_t nStr;
     size_t nInSize;
     size_t nOutSize;
-    size_t nConv;
+    size_t iconv_status = -1;
     size_t nBufferSize;
 
     char* pszUTF8 = NULL;
@@ -1522,7 +1523,7 @@ char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
         nBufferSize = ((nStr * 6) + 1);
         output = (char*) malloc (nBufferSize);
         if (output == NULL) {
-            msSetError(MS_MEMERR, NULL, "msGetEncodedString()");
+            msSetError(MS_MEMERR, NULL, "msConvertWideStringToUTF8()");
             return NULL;
         }
         if (nStr == 0) {
@@ -1536,27 +1537,41 @@ char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
         nOutSize = nBufferSize;
         if ((iconv_t)-1 != cd)
         {  
-           nInSize = sizeof (wchar_t)*nStr;
-           pszUTF8 = output;
-           pwszWide = string;
-           nConv = iconv(cd, (char **)&pwszWide, &nInSize, &pszUTF8, &nOutSize);
-           if ((size_t)-1 != nConv &&  nOutSize != nBufferSize)
-               bconvFailed = MS_FALSE;
-           iconv_close(cd);
+            nInSize = sizeof (wchar_t)*nStr;
+            pszUTF8 = output;
+            pwszWide = string;
+            iconv_status = iconv(cd, (char **)&pwszWide, &nInSize, &pszUTF8, &nOutSize);
+            if ((size_t)-1 == iconv_status) {
+                switch (errno) {
+                    case E2BIG:
+                    errormessage = "There is not sufficient room in buffer";
+                    break;
+                    case EILSEQ:
+                    errormessage = "An invalid multibyte sequence has been encountered in the input";
+                    break;
+                    case EINVAL:
+                    errormessage = "An incomplete multibyte sequence has been encountered in the input";
+                    break;
+                    default:
+                    errormessage = "Unknown";
+                    break;
+                }
+                msSetError(MS_MISCERR, "Unable to convert string in encoding '%s' to UTF8 %s",
+                                       "msConvertWideStringToUTF8()",
+                           encoding,errormessage);
+                iconv_close(cd);
+                msFree(output);
+                return NULL;
+            }
+            iconv_close(cd);
         } else {
             msSetError(MS_MISCERR, "Encoding not supported by libiconv (%s).", 
                                    "msConvertWideStringToUTF8()", 
                                    encoding);
+            msFree(output);
             return NULL;
         }
    
-        if (bconvFailed) {
-            msFree(output);
-            output = NULL;
-            msSetError(MS_MISCERR, "Unable to convert string in encoding '%s' to UTF8",
-                                   "msConvertWideStringToUTF8()",
-                                   encoding);
-        }
     } else {
         /* we were given a NULL wide string, nothing we can do here */
         return NULL;
