@@ -1,11 +1,11 @@
 /******************************************************************************
+ * $Id: mapxbase.c 7418 2008-02-29 00:02:49Z nsavard $
  *
  * Project:  MapServer
- * Purpose:  .dbf access API.
+ * Purpose:  .dbf access API.  Derived from shapelib, and relicensed with 
+ *           permission of Frank Warmerdam (shapelib author).
  * Author:   Steve Lime and the MapServer team.
  *
- * Derived from shapelib, and relicensed with permission.
- *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -32,7 +32,7 @@
 #include <stdlib.h> /* for atof() and atoi() */
 #include <math.h>
 
-MS_CVSID("$Id: mapxbase.c 6600 2007-08-16 14:48:27Z dmorissette $")
+MS_CVSID("$Id: mapxbase.c 7418 2008-02-29 00:02:49Z nsavard $")
 
 /************************************************************************/
 /*                             SfRealloc()                              */
