@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shp2img.c 9518 2009-11-04 13:53:23Z aboudreault $
+ * $Id: shp2img.c 10764 2010-11-25 20:28:32Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Commandline .map rendering utility, mostly for testing.
@@ -30,7 +30,7 @@
 #include "mapserver.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: shp2img.c 9518 2009-11-04 13:53:23Z aboudreault $")
+MS_CVSID("$Id: shp2img.c 10764 2010-11-25 20:28:32Z aboudreault $")
 
 int main(int argc, char *argv[])
 {
@@ -43,7 +43,6 @@ int main(int argc, char *argv[])
   int num_layers=0;
 
   int layer_found=0;
-  char *invalid_layer=NULL;
 
   char *outfile=NULL; /* no -o sends image to STDOUT */
 
@@ -256,25 +255,18 @@ int main(int argc, char *argv[])
       layers = msStringSplit(argv[i+1], ' ', &(num_layers));
 
       for(j=0; j<num_layers; j++) { /* loop over -l */
-        layer_found=0;
-        for(k=0; k<map->numlayers; k++) {
-          if(GET_LAYER(map, k)->name && strcmp(GET_LAYER(map, k)->name, layers[j]) == 0) {
-            layer_found=1;
-            break;
+          layer_found=0;
+          for(k=0; k<map->numlayers; k++) {
+              if(GET_LAYER(map, k)->name && strcmp(GET_LAYER(map, k)->name, layers[j]) == 0) {
+                  layer_found = 1;
+                  break;
+              }
           }
-          else {
-            if (invalid_layer)
-              free(invalid_layer);
-            invalid_layer = strdup(layers[j]);
+          if (layer_found==0) {
+              fprintf(stderr, "Layer (-l) \"%s\" not found\n", layers[j]);
+              msCleanup();
+              exit(0);
           }
-        }
-        if (layer_found==0) {
-          fprintf(stderr, "Layer (-l) %s not found\n", invalid_layer);
-          msCleanup();
-          exit(0);
-        }
-        if (invalid_layer)
-          free(invalid_layer);
       }
 
       for(j=0; j<map->numlayers; j++) {
