@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: shptree.c 8015 2008-11-11 18:14:14Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  Commandline utility to generate .qix shapefile spatial indexes.
@@ -30,7 +31,7 @@
 #include "maptree.h"
 #include <string.h>
 
-MS_CVSID("$Id: shptree.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: shptree.c 8015 2008-11-11 18:14:14Z pramsey $")
 
 char* AddFileSuffix ( const char * Filename, const char * Suffix ) {
   char	*pszFullname, *pszBasename;
@@ -123,7 +124,7 @@ int main(int argc, char *argv[])
       byte_order = MS_NEW_MSB_ORDER; 
   }
     
-  if(msSHPOpenFile(&shapefile, "rb", argv[1]) == -1) {
+  if(msShapefileOpen(&shapefile, "rb", argv[1], MS_TRUE) == -1) {
     fprintf(stdout, "Error opening shapefile %s.\n", argv[1]);
     exit(0);
   }
@@ -148,7 +149,7 @@ int main(int argc, char *argv[])
   /*
   ** Clean things up
   */
-  msSHPCloseFile(&shapefile);
+  msShapefileClose(&shapefile);
 
   return(0);
 }
